/* Project source file */

#include <vcl.h>

#pragma hdrstop

#include "ADOManagerQuery.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
TInputLoginForm* LoginForm;
__fastcall TProcessInfo::TProcessInfo()
{
	Reset();
}
void __fastcall TProcessInfo::Reset()
{
	ID="N/A";
	LoginName="N/A";
	DBName="N/A";
	DBUser="N/A";
	ServerName="N/A";
}
//---------------------------------------------------------------------------
void __fastcall TLoginInfo::Reset()
{
	LoginName="";
	Password="";
	OldPassword="";
	DefaultDatabase="";
	Role=""; //DBRole, Server role, app role
	UserName=""; //DB User name
	Option=false; //Remove, deny, encrypt
}
//---------------------------------------------------------------------------
__fastcall TDBBackupInfo::TDBBackupInfo()
{
	DBFileList=new TTntStringList;
	NewPathList=new TTntStringList;
	DiskFile=true;
	BackupSetNumber=1;
	LogFileName="";
	NewLogPath="";
}
void __fastcall TDBBackupInfo::AddBackupFile(WideString FileName)
{
	DBFileList->Add(FileName);
}
WideString __fastcall TDBBackupInfo::FGetDBFile(int index)
{
	return DBFileList->Strings[index];
}
void __fastcall TDBBackupInfo::DeleteBackupFile(WideString FileName)
{
	DBFileList->Delete(DBFileList->IndexOf(FileName));
}
void __fastcall TDBBackupInfo::AddNewFilePath(WideString FileName)
{
	NewPathList->Add(FileName);
}
WideString __fastcall TDBBackupInfo::FGetNewFilePath(int index)
{
	return NewPathList->Strings[index];
}
void __fastcall TDBBackupInfo::DeleteNewFilePath(WideString FileName)
{
	NewPathList->Delete(NewPathList->IndexOf(FileName));
}
void __fastcall TDBBackupInfo::ResetDBFileList()
{
	DBFileList->Clear();
}
void __fastcall TDBBackupInfo::ResetPathList()
{
	NewPathList->Clear();
}
int __fastcall TDBBackupInfo::FDBFileCount()
{
	return DBFileList->Count;
}
int __fastcall TDBBackupInfo::FNewFilePathCount()
{
	return NewPathList->Count;
}
void __fastcall TDBBackupInfo::Reset()
{
	DiskFile=true;
	BackupSetNumber=1;
	DBFileList->Clear();
	NewPathList->Clear();
	LogFileName="";
	NewLogPath="";
	DeviceName="";
	DBName="";
}
__fastcall TDBBackupInfo::~TDBBackupInfo()
{
	delete DBFileList;
	delete NewPathList;
}
//---------------------------------------------------------------------------
static inline void ValidCtrCheck(TADOSqlManagerQuery *)
{
	new TADOSqlManagerQuery(NULL);
}
//---------------------------------------------------------------------------
__fastcall TADOSqlManagerQuery::TADOSqlManagerQuery(TComponent* Owner)
	: TADOQuery(Owner)
{
	//FDisplayError=true;
	unsigned long size=MAX_COMPUTERNAME_LENGTH;
	FComputerName.SetLength(MAX_COMPUTERNAME_LENGTH);
	FUserName.SetLength(UNLEN);
	GetComputerName(FComputerName.c_str(), &size);
	FComputerName.SetLength(size);
	size=UNLEN;
	GetUserName(FUserName.c_str(), &size);
	FUserName.SetLength(size);
	FUserName=TrimRight(FUserName);
	FLastError="";
	Prepared=true;
	FLastRunError=false;
	//FSqlLogger.OutputMemo=OutputLogMemo;
	//FSqlLogger.WriteLog("SQL Logger started",true);
}
//---------------------------------------------------------------------------
__fastcall TADOSqlManagerQuery::~TADOSqlManagerQuery()
{
	//TADOQuery::~TADOQuery();
	//delete FCurrentProcessInfo;
	FSqlLogger.OutputMemo=NULL;
}
namespace Adomanagerquery
{
	void __fastcall PACKAGE Register()
	{
		TComponentClass classes[1] = {__classid(TADOSqlManagerQuery)};
		RegisterComponents("dbGO", classes, 0);
	}
}
//---------------------------------------------------------------------------
int __fastcall TADOSqlManagerQuery::ExecSQL()
{
	FLastRunError=false;
	if(FQueryLog)
	{
		WriteLog((WideString)"/*SQL execute*/ "+SQL->Text,true);
	}
	try
	{
		return TADOQuery::ExecSQL();
	}
	catch(Exception &e)
	{
		FLastRunError=true;
		FLastError=e.Message;
		if(FDisplayError)
		{
			MessageBox(NULL, FLastError.c_str(),"SQL execute error",MB_ICONERROR|MB_TASKMODAL);
		}
		if(FErrorLog)
		{
			WriteLog((WideString)("/*SQL execute error: "+FLastError+"*/"),true);
		}
		return -1;
	}
}
void _fastcall TADOSqlManagerQuery::Open()
{
	if(Active)
	{
		Close();
	}
	FLastRunError=false;
	if(FQueryLog)
	{
		WriteLog((WideString)"/*SQL open*/ "+SQL->Text,true);
	}
	try
	{
		TADOQuery::Open();
	}
	catch(Exception &e)
	{
		if(AnsiCompareStr(RightStr(e.Message,28),"does not return a result set"))
		{
			FLastRunError=true;
			FLastError=e.Message;
			if(FDisplayError)
			{
				MessageBox(NULL, FLastError.c_str(),"SQL open error",MB_ICONERROR|MB_TASKMODAL);
			}
			if(FErrorLog)
			{
				WriteLog((WideString)("/*SQL open error: "+FLastError+"*/"),true);
			}
		}
	}
}
void __fastcall TADOSqlManagerQuery::WriteLog(WideString Text, bool WriteTime)
{
	FSqlLogger.WriteLog(Text,WriteTime);
}
void __fastcall TADOSqlManagerQuery::LoginFormDestroy()
{
	LoginForm=NULL;
}
WideString __fastcall TADOSqlManagerQuery::ReadDatabaseName()
{
	return FCurrentProcessInfo.DBName;
}
WideString __fastcall TADOSqlManagerQuery::ReadServerName()
{
	return FCurrentProcessInfo.ServerName;
}
WideString __fastcall TADOSqlManagerQuery::ReadLoginName()
{
	return FCurrentProcessInfo.LoginName;
}
WideString __fastcall TADOSqlManagerQuery::ReadUserName()
{
	return FCurrentProcessInfo.DBUser;
}
AnsiString __fastcall TADOSqlManagerQuery::ReadID()
{
	return FCurrentProcessInfo.ID;
}
void __fastcall TADOSqlManagerQuery::SetDatabase(WideString Databasename)
{
	SQL->Text=(WideString)"USE "+Databasename;
	ExecSQL();
	if(!FLastRunError)
		UpdateProcessInfo();
}
//---------------------------------------------------------------------------
void __fastcall TADOSqlManagerQuery::SetSqlLogMemo(TTntMemo* value)
{
	FSqlLogger.OutputMemo=value;
}
void __fastcall TADOSqlManagerQuery::SetSqlLogFileName(AnsiString value)
{
	FSqlLogger.FileName=value;
	if(!ComponentState.Contains(csDesigning))
	{
        FSqlLogger.Initialize();
    }
}
void __fastcall TADOSqlManagerQuery::SetOnWriteLog(TWriteLogEvent value)
{
	FSqlLogger.OnWriteLog=value;
	//FOnWriteLog=value;
}
void __fastcall TADOSqlManagerQuery::SetDisplayError(bool value)
{
	if(FDisplayError!=value)
	{
		FDisplayError=value;
		FSqlLogger.DisplayError=value;
    }
}
bool __fastcall TADOSqlManagerQuery::UpdateProcessInfo()
{
	SQL->Text="SELECT @@SPID AS 'ID', SERVERPROPERTY('ServerName') AS 'ServerName', SYSTEM_USER AS 'LoginName', DB_NAME() AS 'DBName', USER AS 'UserName'";
	Open();
	if(!FLastRunError)
	{
		FCurrentProcessInfo.ID=FieldByName("ID")->AsString;
		FCurrentProcessInfo.LoginName=((TWideStringField*)FieldByName("LoginName"))->Value;
		FCurrentProcessInfo.DBUser=((TWideStringField*)FieldByName("UserName"))->Value;
		FCurrentProcessInfo.DBName=((TWideStringField*)FieldByName("DBName"))->Value;
		FCurrentProcessInfo.ServerName=((TWideStringField*)FieldByName("ServerName"))->Value;
		Close();
		return true;
	}
	else
	{
		FCurrentProcessInfo.Reset();
		return false;
	}
}
bool __fastcall TADOSqlManagerQuery::ServerInfo(TTntStringList* VersionInfo)
{
	SQL->Text="SELECT SERVERPROPERTY('ServerName') As ServerName, ";
	SQL->Add("SERVERPROPERTY('MachineName') As MachineName, ");
	SQL->Add("SERVERPROPERTY('Edition')As Edition, ");
	SQL->Add("SERVERPROPERTY('EngineEdition')As EngineEdition, ");
	SQL->Add("SERVERPROPERTY('ProductVersion') As ProductVersion, ");
	SQL->Add("SERVERPROPERTY('ProductLevel')As ProductLevel, ");
	SQL->Add("SERVERPROPERTY('LicenseType') As LicenseType, ");
	SQL->Add("SERVERPROPERTY('NumLicenses')As NumLincenses, ");
	SQL->Add("SERVERPROPERTY('ProcessID') As ProcessID, ");
	SQL->Add("SERVERPROPERTY('Collation') As Collation, ");
	SQL->Add("SERVERPROPERTY('IsClustered') As Cluster, ");
	SQL->Add("SERVERPROPERTY('IsFullTextInstalled') As FullText, ");
	SQL->Add("SERVERPROPERTY('IsIntegratedSecurityOnly') as IntegratedSecurity, ");
	SQL->Add("SERVERPROPERTY('IsSingleUser')As SingleUser, ");
	SQL->Add("SERVERPROPERTY('IsSyncWithBackup') as SyncWithBackup");
	//LastResult=true;
	VersionInfo->Add("<Version>");
	Open();
	if(!FLastRunError)
	{
		VersionInfo->Add((WideString)"\t<ServerName>"+((TWideStringField*)FieldByName("ServerName"))->Value+(WideString)"</ServerName>");
		VersionInfo->Add((WideString)"\t<MachineName>"+((TWideStringField*)FieldByName("MachineName"))->Value+(WideString)"</MachineName>");
		VersionInfo->Add((WideString)"\t<Edition>"+((TWideStringField*)FieldByName("Edition"))->Value+(WideString)"</Edition>");
		VersionInfo->Add((WideString)"\t<EngineEdition>"+((TWideStringField*)FieldByName("EngineEdition"))->Value+(WideString)"</EngineEdition>");
		VersionInfo->Add((WideString)"\t<ProductVersion>"+((TWideStringField*)FieldByName("ProductVersion"))->Value+(WideString)"</ProductVersion>");
		VersionInfo->Add((WideString)"\t<ProductLevel>"+((TWideStringField*)FieldByName("ProductLevel"))->Value+(WideString)"</ProductLevel>");
		VersionInfo->Add((WideString)"\t<LicenseType>"+((TWideStringField*)FieldByName("LicenseType"))->Value+(WideString)"</LicenseType>");
		VersionInfo->Add((WideString)"\t<NumLincenses>"+((TWideStringField*)FieldByName("NumLincenses"))->Value+(WideString)"</NumLincenses>");
		VersionInfo->Add((WideString)"\t<ProcessID>"+((TWideStringField*)FieldByName("ProcessID"))->Value+(WideString)"</ProcessID>");
		VersionInfo->Add((WideString)"\t<Collation>"+((TWideStringField*)FieldByName("Collation"))->Value+(WideString)"</Collation>");
		VersionInfo->Add((WideString)"\t<Clustered>"+((TWideStringField*)FieldByName("Cluster"))->Value+(WideString)"</Clustered>");
		VersionInfo->Add((WideString)"\t<FullTextSearch>"+((TWideStringField*)FieldByName("FullText"))->Value+(WideString)"</FullTextSearch>");
		VersionInfo->Add((WideString)"\t<IntegratedSecurityOnly>"+((TWideStringField*)FieldByName("IntegratedSecurity"))->Value+(WideString)"</IntegratedSecurityOnly>");
		VersionInfo->Add((WideString)"\t<SingleUserMode>"+((TWideStringField*)FieldByName("SingleUser"))->Value+(WideString)"</SingleUserMode>");
		VersionInfo->Add((WideString)"\t<SyncWithBackup>"+((TWideStringField*)FieldByName("SyncWithBackup"))->Value+(WideString)"</SyncWithBackup>");
		Close();
	}
	VersionInfo->Add("</Version>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetServerOS(TTntStringList *ServerOS)
{
	SQL->Text="EXEC xp_msver";
	ServerOS->Add("<OSInfo>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			AnsiString FieldName=FieldByName("Name")->AsString;
			if(FieldName=="Product ID")
				FieldName="ProductID";
			ServerOS->Add("<"+FieldName+">"+FieldByName("Character_Value")->AsString+"</"+FieldName+">");
			Next();
		}
		Close();
	}
	ServerOS->Add("</OSInfo>");
	return FLastRunError;
}
void __fastcall TADOSqlManagerQuery::Shutdown(bool NoWait)
{
	SQL->Text="SHUTDOWN";
	if(NoWait)
		SQL->Text=SQL->Text+" WITH NOWAIT";
	ExecSQL();
}
//---------------------------------------------------------------------------
int __fastcall TADOSqlManagerQuery::LoginPrompt()
{
	if(LoginForm==NULL)
	{
		LoginForm=new TInputLoginForm(Owner);
	}
	LoginForm->SqlQuery=(TADOQuery*)this;
	return LoginForm->ShowModal();
}
bool __fastcall TADOSqlManagerQuery::GetConnections(TTntStringList* Connections)
{
	SQL->Text=(WideString)"EXEC sp_who";
	Connections->Add("<Connections>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			Connections->Add("\t<Connection>");
			Connections->Add("\t\t<ProcessID>"+FieldByName("spid")->AsString+"</ProcessID>");
			Connections->Add("\t\t<ContextID>"+FieldByName("ecid")->AsString+"</ContextID>");
			Connections->Add("\t\t<Status>"+TrimRight(FieldByName("status")->AsString)+"</Status>");
			Connections->Add((WideString)"\t\t<Login>"+((TWideStringField*)FieldByName("loginame"))->Value+(WideString)"</Login>");
			Connections->Add((WideString)"\t\t<Hostname>"+TrimRight(((TWideStringField*)FieldByName("hostname"))->Value)+(WideString)"</Hostname>");
			Connections->Add("\t\t<BlockingProcess>"+TrimRight(FieldByName("blk")->AsString)+"</BlockingProcess>");
			Connections->Add((WideString)"\t\t<Database>"+ ((TWideStringField*)FieldByName("dbname"))->Value+(WideString)"</Database>");
			Connections->Add("\t\t<Command>"+TrimRight(FieldByName("cmd")->AsString)+"</Command>");
			Connections->Add("\t</Connection>");
			Next();
		}
		Close();
	}
	Connections->Add("</Connections>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::TerminateConnections(TStringList* PIDList)
{
	SQL->Clear();
	for(int i=0;i<PIDList->Count;i++)
	{
		SQL->Add("KILL "+PIDList->Strings[i]);
	}
	ExecSQL();
	return FLastRunError;
}
void __fastcall TADOSqlManagerQuery::Disconnect()
{
	Active=false;
	ConnectionString="";
	WriteLog("User disconnect",true);
	FCurrentProcessInfo.Reset();
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::GetLogins(TTntStringList* LoginList)
{
	SQL->Text="EXEC sp_helplogins";
	LoginList->Add("<Logins>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			LoginList->Add("\t<Login>");
			LoginList->Add((WideString)"\t\t<Name>" + ((TWideStringField*)FieldByName("LoginName"))->Value +(WideString)"</Name>");
			LoginList->Add((WideString)"\t\t<DefaultDatabase>" + ((TWideStringField*)FieldByName("DefDBName"))->Value +(WideString)"</DefaultDatabase>");
			LoginList->Add((WideString)"\t\t<DefLangName>" + ((TWideStringField*)FieldByName("DefLangName"))->Value +(WideString)"</DefLangName>");
			LoginList->Add((WideString)"\t\t<IsUser>"+TrimRight(LowerCase(FieldByName("AUser")->AsString))+(WideString)"</IsUser>");
			LoginList->Add((WideString)"\t\t<IsRemote>"+TrimRight(LowerCase(FieldByName("ARemote")->AsString))+(WideString)"</IsRemote>");
			LoginList->Add("\t</Login>");
			Next();
		}
		Close();
	}
	LoginList->Add("</Logins>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetLoginProperties(TTntStringList* LoginProperties, WideString LoginName, bool DBAccess)
{
	SQL->Text=(WideString)"EXEC sp_helplogins N'"+LoginName+(WideString)"'";
	LoginProperties->Add("<Login>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			LoginProperties->Add((WideString)"\t<Name>" + ((TWideStringField*)FieldByName("LoginName"))->Value +(WideString)"</Name>");
			LoginProperties->Add((WideString)"\t<DefaultDatabase>" + ((TWideStringField*)FieldByName("DefDBName"))->Value +(WideString)"</DefaultDatabase>");
			LoginProperties->Add((WideString)"\t<DefLangName>" + ((TWideStringField*)FieldByName("DefLangName"))->Value +(WideString)"</DefLangName>");
			LoginProperties->Add((WideString)"\t<IsUser>"+TrimRight(LowerCase(FieldByName("AUser")->AsString))+(WideString)"</IsUser>");
			LoginProperties->Add((WideString)"\t<IsRemote>"+TrimRight(LowerCase(FieldByName("ARemote")->AsString))+(WideString)"</IsRemote>");
			Next();
		}
		if(DBAccess)
		{
			int temp;
			Recordset=NextRecordset(temp);
			LoginProperties->Add("\t<Databases>");
			WideString OldDatabase="";
			WideString CurrentDatabase="N/A";
			while(!Eof)
			{
				CurrentDatabase=((TWideStringField*)FieldByName("DBName"))->Value;
				if(CurrentDatabase!=OldDatabase)
				{
					LoginProperties->Add((WideString)"\t\t<"+CurrentDatabase+(WideString)">");
				}
				if(TrimRight(FieldByName("UserOrAlias")->AsString)=="User")
				{
					LoginProperties->Add((WideString)"\t\t\t<UserName>" + ((TWideStringField*)FieldByName("UserName"))->Value+ (WideString)"</UserName>");
					LoginProperties->Add((WideString)"\t\t</"+CurrentDatabase+(WideString)">");
				}
				else
				{
					LoginProperties->Add((WideString)"\t\t\t<DBRole>" + ((TWideStringField*)FieldByName("UserName"))->Value+ (WideString)"</DBRole>");
				}
				OldDatabase=CurrentDatabase;
				Next();
			}
			LoginProperties->Add("\t</Databases>");
		}
		Close();
	}
	LoginProperties->Add("</Login>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::CreateLogins(TLoginInfo* LoginInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_addlogin N'"+LoginInfos[i].LoginName+(WideString)"', N'"+LoginInfos[i].Password+(WideString)"'");
		if(LoginInfos[i].DefaultDatabase.Length())
		{
			SQL->Add((WideString)", N'"+LoginInfos[i].DefaultDatabase+(WideString)"'");
		}
		if(LoginInfos[i].Role.Length())
		{
			SQL->Add((WideString)"EXEC sp_addsrvrolemember N'"+LoginInfos[i].LoginName+(WideString)"', '"+LoginInfos[i].Role+(WideString)"'");
		}
	}
	ExecSQL();
	return FLastRunError;
}
char* __fastcall TADOSqlManagerQuery::GetLoginPassword(WideString Name)
{
	if(DatabaseName!=(WideString)"master")
	{
		DatabaseName="master";
	}
	SQL->Text=(WideString)"SELECT CONVERT(VARBINARY(32), password) as [Password] FROM syslogins WHERE name = N'"+Name+(WideString)"'";
	Open();
	if(FLastRunError)
	{
        return NULL;
    }
	char Buffer[34];
	char Password[66];
	if(RecordCount)
	{
		FieldByName("Password")->GetData(Buffer,true);
		BinToHex(&Buffer[2], &Password[2],32);
	}
	Close();
	Password[0]='0';
	Password[1]='x';
	return &Password[0];
}
bool __fastcall TADOSqlManagerQuery::ChangePasswords(TLoginInfo* LoginInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_password N'"+LoginInfos[i].OldPassword+(WideString)"', N'"+LoginInfos[i].Password+(WideString)"', N'"+LoginInfos[i].LoginName+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteLogins(TLoginInfo* LoginInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_droplogin N'"+LoginInfos[i].LoginName+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetAllLoginPassword(TTntStringList* Logins)
{
	if(DatabaseName!=(WideString)"master")
	{
		DatabaseName="master";
	}
	SQL->Text=(WideString)"SELECT name, CONVERT(VARBINARY(32), password) as [Password] FROM syslogins";
	Logins->Add("<Logins>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			char Buffer[34];
			char Password[66];
			FieldByName("Password")->GetData(Buffer,true);
			BinToHex(&Buffer[2], &Password[2],32);
			Password[0]='0';
			Password[1]='x';
			AnsiString Pass=&Password[0];
			Pass=TrimRight(Pass);
			Logins->Add("\t<Login>");
			Logins->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("name"))->Value+(WideString)"</Name>");
			Logins->Add((WideString)"\t\t<Password>"+(WideString)TrimRight(Pass)+(WideString)"</Password>");
			Logins->Add("\t</Login>");
			Next();
		}
		Close();
	}
	Logins->Add("</Logins>");
	return FLastRunError;
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::CreateUsers(TLoginInfo* LoginInfos, int Count)
{
   SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_grantdbaccess N'"+LoginInfos[i].LoginName+(WideString)"', N'"+LoginInfos[i].UserName+(WideString)"'");
		if(LoginInfos[i].Role.Length())
		{
			SQL->Add((WideString)"EXEC sp_addrolemember N'"+LoginInfos[i].Role+(WideString)"', N'"+LoginInfos[i].UserName+(WideString)"'");
		}
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetUsers(TTntStringList* UserList)
{
	SQL->Text="EXEC sp_helpuser";
	UserList->Add("<Users>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			UserList->Add("\t<User>");
			UserList->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("UserName"))->Value+(WideString)"</Name>");
			UserList->Add((WideString)"\t\t<DBRole>"+((TWideStringField*)FieldByName("GroupName"))->Value+(WideString)"</DBRole>");
			UserList->Add((WideString)"\t\t<LoginName>"+((TWideStringField*)FieldByName("LoginName"))->Value+(WideString)"</LoginName>");
			UserList->Add("\t</User>");
			Next();
		}
		Close();
	}
	UserList->Add("</Users>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ChangeUsersLogins(TLoginInfo* LoginInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_change_users_login 'Update_One', N'"+LoginInfos[i].UserName+(WideString)"', N'"+LoginInfos[i].LoginName+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteUsers(TLoginInfo* LoginInfos, int Count)
{
   SQL->Clear();
   for(int i=0;i<Count;i++)
   {
	  SQL->Add((WideString)"EXEC sp_revokedbaccess N'"+LoginInfos[i].UserName+(WideString)"'");
   }
	ExecSQL();
	return FLastRunError;
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::HasAccessDatabases(TTntStringList* DatabaseList)
{
	SQL->Text="EXEC sp_MShasdbaccess";
	DatabaseList->Add("<Databases>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			DatabaseList->Add("\t<Database>");
			DatabaseList->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("dbname"))->Value+(WideString)"</Name>");
			DatabaseList->Add((WideString)"\t\t<Owner>"+((TWideStringField*)FieldByName("owner"))->Value+(WideString)"</Owner>");
			DatabaseList->Add("\t\t<DboOnly>"+FieldByName("DboOnly")->AsString+"</DboOnly>");
			DatabaseList->Add("\t\t<ReadOnly>"+FieldByName("ReadOnly")->AsString+"</ReadOnly>");
			DatabaseList->Add("\t\t<SingleUser>"+FieldByName("SingleUser")->AsString+"</SingleUser>");
			DatabaseList->Add("\t\t<Detached>"+FieldByName("Detached")->AsString+"</Detached>");
			DatabaseList->Add("\t\t<Suspect>"+FieldByName("Suspect")->AsString+"</Suspect>");
			DatabaseList->Add("\t\t<Offline>"+FieldByName("Offline")->AsString+"</Offline>");
			DatabaseList->Add("\t\t<Inload>"+FieldByName("Inload")->AsString+"</Inload>");
			DatabaseList->Add("\t\t<EmergencyMode>"+FieldByName("EmergencyMode")->AsString+"</EmergencyMode>");
			DatabaseList->Add("\t\t<StandBy>"+FieldByName("StandBy")->AsString+"</StandBy>");
			DatabaseList->Add("\t\t<ShutDown>"+FieldByName("ShutDown")->AsString+"</ShutDown>");
			DatabaseList->Add("\t\t<InRecovery>"+FieldByName("InRecovery")->AsString+"</InRecovery>");
			DatabaseList->Add("\t\t<NotRecovered>"+FieldByName("NotRecovered")->AsString+"</NotRecovered>");
			DatabaseList->Add("\t</Database>");
			Next();
		}
		Close();
	}
	DatabaseList->Add("</Databases>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ListDatabases(TTntStringList* DatabaseList)
{
	SQL->Text="EXEC sp_databases";
	DatabaseList->Add("<Databases>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			DatabaseList->Add("\t<Database>");
			DatabaseList->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("DATABASE_NAME"))->Value+(WideString)"</Name>");
			DatabaseList->Add((WideString)"\t\t<Size>"+TrimLeft(FieldByName("DATABASE_SIZE")->AsString)+" KB"+(WideString)"</Size>");
			DatabaseList->Add("\t</Database>");
			Next();
		}
		Close();
	}
	DatabaseList->Add("</Databases>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetAllDatabases(TTntStringList* DatabaseList, bool Status)
{
	SQL->Text="EXEC sp_helpdb";
	DatabaseList->Add("<Databases>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			DatabaseList->Add("\t<Database>");
			DatabaseList->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("name"))->Value+(WideString)"</Name>");
			DatabaseList->Add((WideString)"\t\t<Size>"+TrimLeft(FieldByName("db_size")->AsString)+(WideString)"</Size>");
			DatabaseList->Add((WideString)"\t\t<Owner>"+((TWideStringField*)FieldByName("owner"))->Value+(WideString)"</Owner>");
			DatabaseList->Add((WideString)"\t\t<DateCreated>"+FieldByName("created")->AsString+(WideString)"</DateCreated>");
			DatabaseList->Add((WideString)"\t\t<Level>"+FieldByName("compatibility_level")->AsString+(WideString)"</Level>");
			if(Status)
			{
				DatabaseList->Add((WideString)"\t\t<Status>"+FieldByName("status")->AsString+(WideString)"</Status>");
			}
			DatabaseList->Add("\t</Database>");
			Next();
		}
		Close();
	}
	DatabaseList->Add("</Databases>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::CreateDatabases(TTntStringList *DatabaseNames)
{
	SQL->Clear();
	for(int i=0;i<DatabaseNames->Count;i++)
	{
		SQL->Add((WideString)"CREATE DATABASE ["+DatabaseNames->Strings[i]+(WideString)"]");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteDatabases(TTntStringList *DatabaseNames)
{
	SQL->Text=(WideString)"DROP DATABASE ["+DatabaseNames->Strings[0]+(WideString)"]";
	for(int i=1;i<DatabaseNames->Count;i++)
	{
		SQL->Add((WideString)", ["+DatabaseNames->Strings[i]+(WideString)"]");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetDBProperties(TTntStringList* DBProperties, WideString DBName, bool DBFiles)
{
	SQL->Text=(WideString)"EXEC sp_helpdb N'"+DBName+(WideString)"'";
	DBProperties->Add("<DBProperties>");
	Open();
	if(!FLastRunError)
	{
		DBProperties->Add((WideString)"\t<Name>"+((TWideStringField*)FieldByName("name"))->Value+(WideString)"</Name>");
		DBProperties->Add("\t<Size>"+TrimLeft(FieldByName("db_size")->AsString)+(WideString)"</Size>");
		DBProperties->Add((WideString)"\t<Owner>"+((TWideStringField*)FieldByName("owner"))->Value+(WideString)"</Owner>");
		DBProperties->Add("\t<DateCreated>"+FieldByName("created")->AsString+(WideString)"</DateCreated>");
		DBProperties->Add("\t<Level>"+FieldByName("compatibility_level")->AsString+(WideString)"</Level>");
		DBProperties->Add("\t<Status>"+FieldByName("status")->AsString+(WideString)"</Status>");
		if(DBFiles)
		{
			int temp;
			Recordset=NextRecordset(temp);
			DBProperties->Add("\t<DBFiles>");
			while(!Eof)
			{
				DBProperties->Add("\t\t<File>");
				DBProperties->Add((WideString)"\t\t\t<Name>"+TrimRight(((TWideStringField*)FieldByName("name"))->Value)+(WideString)"</Name>");
				DBProperties->Add((WideString)"\t\t\t<FileGroup>"+FieldByName("filegroup")->AsString+(WideString)"</FileGroup>");
				DBProperties->Add((WideString)"\t\t\t<Size>"+FieldByName("size")->AsString+(WideString)"</Size>");
				DBProperties->Add((WideString)"\t\t\t<Path>"+TrimRight(((TWideStringField*)FieldByName("filename"))->Value)+(WideString)"</Path>");
				DBProperties->Add((WideString)"\t\t\t<MaxSize>"+FieldByName("maxsize")->AsString+(WideString)"</MaxSize>");
				DBProperties->Add((WideString)"\t\t\t<Growth>"+FieldByName("growth")->AsString+(WideString)"</Growth>");
				DBProperties->Add((WideString)"\t\t\t<Usage>"+TrimRight(FieldByName("usage")->AsString)+(WideString)"</Usage>");
				DBProperties->Add("\t\t</File>");
				Next();
			}
			DBProperties->Add("\t</DBFiles>");
		}
		Close();
	}
	DBProperties->Add("</DBProperties>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DetachDB(WideString DBName, bool SkipCheck)
{
   SQL->Text=(WideString)"EXEC sp_detach_db ["+ DBName+(WideString)"], ";
   if(SkipCheck)
   {
	  SQL->Text=SQL->Text+(WideString)"true";
   }
   else
   {
	  SQL->Text=SQL->Text+(WideString)"false";
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::AttachDB(WideString DBName, TTntStringList* FileList)
{
   SQL->Text=(WideString)"EXEC sp_attach_db ["+DBName+(WideString)"]";
   for(int i=0;i<FileList->Count;i++)
   {
	  SQL->Add((WideString)", N'"+FileList->Strings[i]+(WideString)"'");
	  if(i==15)
		 break;
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::AttachDBSingleFile(WideString DBName, WideString FileName)
{
	SQL->Text=(WideString)"EXEC sp_attach_single_file_db ["+DBName+(WideString)"], N'"+FileName+(WideString)"'";
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ChangeDBOwner(WideString NewOwner, bool RemapAlias)
{
   WideString Remap;
   if(RemapAlias)
   {
	  Remap="'true'";
   }
   else
   {
	  Remap="'false'";
   }
   SQL->Text=(WideString)"EXEC sp_changedbowner N'"+NewOwner+(WideString)"', "+Remap;
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ShrinkDBs(TTntStringList* DBNames, int* Percents)
{
   SQL->Clear();
   for(int i=0;i<DBNames->Count;i++)
   {
	  SQL->Add((WideString)"DBCC SHRINKDATABASE (N'"+DBNames->Strings[i]+(WideString)("', "+IntToStr(Percents[i])+")"));
   }
	ExecSQL();
	return FLastRunError;
}
//--------------------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::BackupDBFiles(TDBBackupInfo* BackupInfo)
{
	SQL->Text=(WideString)"BACKUP DATABASE ["+BackupInfo->DBName+ (WideString)"] ";
	for (int i=0;i<BackupInfo->DBFileCount;i++)
	{
		SQL->Add((WideString)"FILE=N'"+BackupInfo->GetDBFile[i]+(WideString)"' ");
		if(i<BackupInfo->DBFileCount-1)
		{
			SQL->Add((WideString)", ");
		}
	}
	if(BackupInfo->DiskFile)
	{
		SQL->Add((WideString)" TO DISK=N'"+BackupInfo->DeviceName+(WideString)"'");
	}
	else
	{
		SQL->Add((WideString)" TO ["+BackupInfo->DeviceName+(WideString)"]");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::AddBackupDevice(AnsiString Type, WideString DevName, WideString Path)
{
	SQL->Text=(WideString)"EXEC sp_addumpdevice '"+Type+(WideString)"', '"+DevName+(WideString)"', '"+Path+(WideString)"'";
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteBackupDevice(WideString DevName, bool DeleteFile)
{
	SQL->Text=(WideString)"EXEC sp_dropdevice '"+DevName+(WideString)"'";
	if(DeleteFile)
	{
		SQL->Text=SQL->Text+(WideString)", 'DELFILE'";
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::IsDeviceExist(WideString DevName)
{
	if(FCurrentProcessInfo.DBName!=(WideString)"master")
	{
		DatabaseName="master";
	}
	SQL->Text=(WideString)"Select name from sysdevices where name=N'"+DevName+(WideString)"'";
	Open();
	if(!FLastRunError)
	{
		bool Result=RecordCount;
		Close();
		return Result;
	}
	else
	{
		return FLastRunError;
	}
}
bool __fastcall TADOSqlManagerQuery::GetBackupDevices(TTntStringList* DeviceList)
{
	DeviceList->Add("<BackupDevices>");
	if(FCurrentProcessInfo.DBName!=(WideString)"master")
	{
		DatabaseName="master";
	}
	SQL->Text="Select * from sysdevices where cntrltype<>0";
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
				DeviceList->Add("\t<Device>");
				DeviceList->Add((WideString)"\t\t<Name>"+TrimRight(((TWideStringField*)FieldByName("name"))->Value)+(WideString)"</Name>");
				DeviceList->Add("\t\t<Size>"+TrimRight(FieldByName("size")->AsString)+"</Size>");
				DeviceList->Add("\t\t<Status>"+TrimRight(FieldByName("status")->AsString)+"</Status>");
				DeviceList->Add("\t\t<ControlType>"+TrimRight(FieldByName("cntrltype")->AsString)+"</ControlType>");
				DeviceList->Add((WideString)"\t\t<PhysicalName>"+TrimRight(((TWideStringField*)FieldByName("phyname"))->Value)+(WideString)"</PhysicalName>");
				DeviceList->Add("\t</Device>");
				Next();
		}
		Close();
	}
	DeviceList->Add("</BackupDevices>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::TruncateDBLogs(TTntStringList* DBNames)
{
	SQL->Clear();
	for(int i=0;i<DBNames->Count;i++)
	{
		SQL->Add((WideString)"BACKUP LOG ["+DBNames->Strings[i]+(WideString)"] WITH TRUNCATE_ONLY");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetBackupFiles(TTntStringList* Files, WideString DevName, bool DiskFile, int BackupSet)
{
   SQL->Text=(WideString)"RESTORE FILELISTONLY FROM ";
   if(DiskFile)
   {
	  SQL->Add((WideString)"DISK=N'"+DevName+(WideString)"'");
   }
   else
   {
	  SQL->Add((WideString)"["+DevName+(WideString)"]");
   }
   SQL->Add((WideString)(" WITH FILE="+IntToStr(BackupSet)));
	Open();
	Files->Add("<BackupFiles>");
	if(!FLastRunError)
	{
		while(!Eof)
		{
			Files->Add("\t<BackupFile>");
			Files->Add((WideString)"\t\t<Name>"+TrimRight(((TWideStringField*)FieldByName("LogicalName"))->Value)+(WideString)"</Name>");
			Files->Add((WideString)"\t\t<Path>"+TrimRight(((TWideStringField*)FieldByName("PhysicalName"))->Value)+(WideString)"</Path>");
			Files->Add((WideString)"\t\t<Type>"+FieldByName("Type")->AsString+(WideString)"</Type>");
			Files->Add((WideString)"\t\t<FileGroup>"+FieldByName("FileGroupName")->AsString+(WideString)"</FileGroup>");
			Files->Add((WideString)"\t\t<Size>"+FieldByName("Size")->AsString+(WideString)"</Size>");
			Files->Add((WideString)"\t\t<MaxSize>"+FieldByName("MaxSize")->AsString+(WideString)"</MaxSize>");
			Files->Add("\t</BackupFile>");
			Next();
		}
		Close();
	}
	Files->Add("</BackupFiles>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::RestoreDBFiles(TDBBackupInfo* BackupInfo)//WideString DBName, TTntStringList* Files, WideString DevName, bool DiskFile, TTntStringList* MoveList, int BackupSet)
{
	SQL->Text=(WideString)"RESTORE DATABASE ["+BackupInfo->DBName+ (WideString)"] ";
	for (int i=0;i<BackupInfo->DBFileCount;i++)
	{
		SQL->Add((WideString)"FILE=N'"+BackupInfo->GetDBFile[i]+(WideString)"' ");
		if(i<BackupInfo->DBFileCount-1)
		{
			SQL->Add((WideString)", ");
		}
	}
	if(BackupInfo->DiskFile)
	{
		SQL->Add((WideString)" FROM DISK=N'"+BackupInfo->DeviceName+(WideString)"'");
	}
	else
	{
		SQL->Add((WideString)" FROM ["+BackupInfo->DeviceName+(WideString)"]");
	}
	SQL->Add((WideString)(" WITH FILE="+IntToStr(BackupInfo->BackupSetNumber)));
	for(int i=0;i<BackupInfo->NewFilePathCount;i++)
	{
		SQL->Add((WideString)", MOVE N'"+BackupInfo->GetDBFile[i]+(WideString)"' TO N'"+BackupInfo->GetNewFilePath[i]+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::RestoreDBLog(TDBBackupInfo* BackupInfo)
{
	SQL->Text=(WideString)"RESTORE LOG ["+BackupInfo->DBName+(WideString)"]";
	if(BackupInfo->DiskFile)
	{
		SQL->Add((WideString)" FROM DISK=N'"+BackupInfo->DeviceName+(WideString)"'");
	}
	else
	{
		SQL->Add((WideString)" FROM ["+BackupInfo->DeviceName+(WideString)"]");
	}
	SQL->Add((WideString)(" WITH FILE="+IntToStr(BackupInfo->BackupSetNumber)));
	if(BackupInfo->NewLogPath.Length())
	{
		SQL->Add((WideString)", MOVE N'"+BackupInfo->LogFileName+ (WideString)"' TO N'"+BackupInfo->NewLogPath+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
//--------------------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::SetPermissions(TPermissionInfo* PermissionInfos, int Count)//TTntStringList* Users, TTntStringList* Tables, TTntStringList* Permissions, bool* Deny)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
	  if(!PermissionInfos[i].Deny)
		 SQL->Add((WideString)"GRANT ");
	  else
		 SQL->Add((WideString)"DENY ");
	  SQL->Add(PermissionInfos[i].Permission+(WideString)" ON ["+PermissionInfos[i].Object);
	  SQL->Add((WideString)"] TO ");
	  SQL->Add((WideString)"["+PermissionInfos[i].UserOrGroup+(WideString)"]");
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ChangeObjectOwner(WideString Object, WideString NewOwner)
{
	SQL->Text=(WideString)"EXEC sp_changeobjectowner N'"+Object+(WideString)"', '"+NewOwner+(WideString)"'";
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetPermissions(WideString Object, WideString User, TTntStringList* Permissions)
{
	SQL->Text=(WideString)"EXEC sp_helprotect N'["+Object+(WideString)"]', ["+User+(WideString)"], NULL, 'o'";
	Open();
	Permissions->Add("<Permissions>");
	if(!FLastRunError)
	{
		while(!Eof)
		{
			Permissions->Add("\t<Permission>");
			Permissions->Add((WideString)"\t\t<Owner>"+((TWideStringField*)FieldByName("Owner"))->Value+(WideString)"</Owner>");
			Permissions->Add((WideString)"\t\t<Object>"+((TWideStringField*)FieldByName("Object"))->Value+(WideString)"</Object>");
			Permissions->Add((WideString)"\t\t<Grantee>"+((TWideStringField*)FieldByName("Grantee"))->Value+(WideString)"</Grantee>");
			Permissions->Add((WideString)"\t\t<Grantor>"+((TWideStringField*)FieldByName("Grantor"))->Value+(WideString)"</Grantor>");
			Permissions->Add("\t\t<Type>"+TrimRight(FieldByName("ProtectType")->AsString) +"</Type>");
			Permissions->Add("\t\t<Action>"+FieldByName("Action")->AsString+"</Action>");
			Permissions->Add((WideString)"\t\t<Column>"+((TWideStringField*)FieldByName("Column"))->Value+(WideString)"</Column>");
			Permissions->Add("</Permission>");
			Next();
		}
		Close();
	}
	Permissions->Add("</Permissions>");
	return FLastRunError;
}
//--------------------------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::GrantWindowLogins(TLoginInfo* AccountInfos, int Count)
{
   SQL->Clear();
   for(int i=0;i<Count;i++)
   {
	  if(AccountInfos[i].Option)
	  {
		 SQL->Add((WideString)"EXEC sp_denylogin N'"+AccountInfos[i].LoginName+(WideString)"'");
	  }
	  else
	  {
		SQL->Add((WideString)"EXEC sp_grantlogin N'"+AccountInfos[i].LoginName+(WideString)"'");
	  }
	  if(AccountInfos[i].Role.Length())
	  {
		 SQL->Add((WideString)"EXEC sp_addsrvrolemember N'"+AccountInfos[i].LoginName+(WideString)"', '"+AccountInfos[i].Role+(WideString)"'");
	  }
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::RemoveWindowLogins(TLoginInfo* AccountInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_revokelogin N'"+AccountInfos[i].LoginName+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::GetServerRoles(TTntStringList* ServerRoles)
{
	SQL->Text=(WideString)"EXEC sp_helpsrvrole";
	ServerRoles->Add("<ServerRoles>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			ServerRoles->Add("\t<ServerRole>");
			ServerRoles->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("ServerRole"))->Value+(WideString)"</Name>");
			ServerRoles->Add((WideString)"\t\t<Description>"+((TWideStringField*)FieldByName("Description"))->Value+(WideString)"</Description>");
			ServerRoles->Add("\t</ServerRole>");
			Next();
		}
		Close();
	}
	ServerRoles->Add("</ServerRoles>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::SetServerRoleMembers(TLoginInfo* AccountInfos, int Count)//TTntStringList* Members, TTntStringList* ServerRoles, bool* Removes)
{
   SQL->Clear();
   for(int i=0;i<Count;i++)
   {
	  if(AccountInfos[i].Option)
	  {
		 SQL->Add((WideString)"EXEC sp_dropsrvrolemember N'"+AccountInfos[i].LoginName+(WideString)"', '"+AccountInfos[i].Role+"'");
	  }
	  else
	  {
		 SQL->Add((WideString)"EXEC sp_addsrvrolemember N'"+AccountInfos[i].LoginName+(WideString)"', '"+AccountInfos[i].Role+"'");
	  }
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetServerRoleMembers(WideString ServerRole, TTntStringList* Members)
{
	SQL->Text="EXEC sp_helpsrvrolemember N'"+ServerRole+"'";
	Members->Add("<Members>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			Members->Add((WideString)"\t<Name>" + ((TWideStringField*)FieldByName("MemberName"))->Value + (WideString)"</Name>");
			Next();
		}
		Close();
	}
	Members->Add("</Members>");
	return FLastRunError;
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::GetDBRoles(TTntStringList* DBRoles)
{
	SQL->Text="EXEC sp_helprole";
	DBRoles->Add("<DBRoles>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			DBRoles->Add("\t<Role>");
			DBRoles->Add((WideString)"\t\t<Name>"+((TWideStringField*)FieldByName("RoleName"))->Value+(WideString)"</Name>");
			DBRoles->Add("\t\t<RoleID>"+FieldByName("RoleId")->AsString+"</RoleID>");
			DBRoles->Add("\t\t<AppRole>"+FieldByName("IsAppRole")->AsString+"/<AppRole>");
			DBRoles->Add("\t</Role>");
			Next();
		}
		Close();
	}
	DBRoles->Add("</DBRoles>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetDBRoleMembers(WideString DBRole, TTntStringList* Members)
{
	SQL->Text=(WideString)"EXEC sp_helprolemember N'"+DBRole+(WideString)"'";
	Members->Add("<Members>");
	Open();
	if(!FLastRunError)
	{
		while(!Eof)
		{
			Members->Add((WideString)"\t<Name>" +((TWideStringField*)FieldByName("MemberName"))->Value+(WideString)"</Name>");
			Next();
		}
		Close();
	}
	Members->Add("</Members>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::SetDBRoleMembers(TLoginInfo* AccountInfos, int Count)//TTntStringList* Members, TTntStringList* DBRoles, bool* Removes)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		if(AccountInfos[i].Option)
		{
			SQL->Add((WideString)"EXEC sp_droprolemember N'"+AccountInfos[i].Role+(WideString)"', N'"+AccountInfos[i].UserName+"'");
		}
		else
		{
			SQL->Add((WideString)"EXEC sp_addrolemember N'"+AccountInfos[i].Role+(WideString)"', N'"+AccountInfos[i].UserName+"'");
		}
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::CreateDBRoles(TLoginInfo* AccountInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_addrole N'"+AccountInfos[i].Role+(WideString)"'");
		if(AccountInfos[i].LoginName.Length())
		{
			SQL->Add((WideString)", N'"+AccountInfos[i].LoginName+(WideString)"'");
		}
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteDBRoles(TTntStringList* DBRoles)
{
   SQL->Clear();
   for(int i=0;i<DBRoles->Count;i++)
   {
	  SQL->Add((WideString)"EXEC sp_droprole N'"+DBRoles->Strings[i]+(WideString)"'");
   }
	ExecSQL();
	return FLastRunError;
}
//---------------------------------------------------------------------------
bool __fastcall TADOSqlManagerQuery::CreateAppRoles(TLoginInfo* AppRoleInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_addapprole N'"+AppRoleInfos[i].Role+(WideString)"'");
		if(AppRoleInfos[i].Password.Length())
		{
			SQL->Add((WideString)", N'"+AppRoleInfos[i].Password+(WideString)"'");
		}
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::DeleteAppRoles(TTntStringList* AppRoles)
{
   SQL->Clear();
   for(int i=0;i<AppRoles->Count;i++)
   {
	  SQL->Add((WideString)"EXEC sp_dropapprole N'"+AppRoles->Strings[i]+(WideString)"'");
   }
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::SetAppRole(TLoginInfo *AppRoleInfo)
{
	SQL->Text="(WideString)EXEC sp_setapprole '"+AppRoleInfo->Role+(WideString)"', {Encrypt N '"+AppRoleInfo->Password+(WideString)"'}";
	if(AppRoleInfo->Option)
	{
		SQL->Text=SQL->Text+(WideString)", 'odbc'";
	}
	ExecSQL();
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::ChangeAppRolePassword(TLoginInfo *AppRoleInfos, int Count)
{
	SQL->Clear();
	for(int i=0;i<Count;i++)
	{
		SQL->Add((WideString)"EXEC sp_approlepassword N'"+AppRoleInfos[i].Role+(WideString)"', '"+AppRoleInfos[i].Password+(WideString)"'");
	}
	ExecSQL();
	return FLastRunError;
}
/* Retrieves all table names from current database.
   
   After return TableInfos will contain XML data in the
   following format:
     * <c>\<Tables\></c>
       * <c>\<Table\></c>
         * <c>\<Name\>Table name\</Name\></c>
         * <c>\<Owner\>Table's owner\</Owner\></c>
         * <c>\<Type\>Table type\</Type\></c>
       * <c>\<Table\></c>
     * <c>\</Tables\></c>                               */
bool __fastcall TADOSqlManagerQuery::GetAllTables(TTntStringList* TableInfos)
{
	SQL->Text="EXEC sp_tables";
	Open();
	TableInfos->Add("<Tables>");
	if(!FLastRunError)
	{
		while(!Eof)
		{

			TableInfos->Add("\t<Table>");
			TableInfos->Add((WideString)"\t\t<Name>" +((TWideStringField*)FieldByName("TABLE_NAME"))->Value+(WideString)"</Name>");
			TableInfos->Add((WideString)"\t\t<Owner>" +((TWideStringField*)FieldByName("TABLE_OWNER"))->Value+(WideString)"</Owner>");
			TableInfos->Add("\t\t<Type>" + FieldByName("TABLE_TYPE")->AsString+"</Type>");
			TableInfos->Add("\t</Table>");
			Next();
		}
		Close();
	}
	TableInfos->Add("</Tables>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetUserTables(TTntStringList* TableInfos)
{
	SQL->Text="EXEC sp_tables @table_type = \"'TABLE'\"";
	Open();
	TableInfos->Add("<Tables>");
	if(!FLastRunError)
	{
		while(!Eof)
		{

			TableInfos->Add("\t<Table>");
			TableInfos->Add((WideString)"\t\t<Name>" +((TWideStringField*)FieldByName("TABLE_NAME"))->Value+(WideString)"</Name>");
			TableInfos->Add((WideString)"\t\t<Owner>" +((TWideStringField*)FieldByName("TABLE_OWNER"))->Value+(WideString)"</Owner>");
			TableInfos->Add("\t\t<Type>" +FieldByName("TABLE_TYPE")->AsString+"</Type>");
			TableInfos->Add("\t</Table>");
			Next();
		}
		Close();
	}
	TableInfos->Add("</Tables>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetSystemTables(TTntStringList* TableInfos)
{
	SQL->Text="EXEC sp_tables @table_type = \"'SYSTEM TABLE'\"";
	Open();
	TableInfos->Add("<Tables>");
	if(!FLastRunError)
	{
		while(!Eof)
		{

			TableInfos->Add("\t<Table>");
			TableInfos->Add((WideString)"\t\t<Name>" +((TWideStringField*)FieldByName("TABLE_NAME"))->Value+(WideString)"</Name>");
			TableInfos->Add((WideString)"\t\t<Owner>" +((TWideStringField*)FieldByName("TABLE_OWNER"))->Value+(WideString)"</Owner>");
			TableInfos->Add("\t\t<Type>" +FieldByName("TABLE_TYPE")->AsString+"</Type>");
			TableInfos->Add("\t</Table>");
			Next();
		}
		Close();
	}
	TableInfos->Add("</Tables>");
	return FLastRunError;
}
bool __fastcall TADOSqlManagerQuery::GetViewTables(TTntStringList* TableInfos)
{
	SQL->Text="EXEC sp_tables @table_type = \"'VIEW'\"";
	Open();
	TableInfos->Add("<Tables>");
	if(!FLastRunError)
	{
		while(!Eof)
		{

			TableInfos->Add("\t<Table>");
			TableInfos->Add((WideString)"\t\t<Name>" +((TWideStringField*)FieldByName("TABLE_NAME"))->Value+(WideString)"</Name>");
			TableInfos->Add((WideString)"\t\t<Owner>" +((TWideStringField*)FieldByName("TABLE_OWNER"))->Value+(WideString)"</Owner>");
			TableInfos->Add("\t\t<Type>" +FieldByName("TABLE_TYPE")->AsString+"</Type>");
			TableInfos->Add("\t</Table>");
			Next();
		}
		Close();
	}
	TableInfos->Add("</Tables>");
	return FLastRunError;
}
